import java.awt.image.BufferedImage;
import javax.swing.JPanel;

/**
 * Hicolor surface class implementation
 */
class Surface {

  public Surface( int a_Width, int a_Height, JPanel panel ) {
    m_Width = a_Width;
    m_Height = a_Height;
    m_Buffer = (BufferedImage)(panel.createImage(a_Width,a_Height));
  }

  // member data access
  public BufferedImage GetBuffer() {
    return m_Buffer;
  }
  public int GetWidth() {
    return m_Width;
  }
  public int GetHeight() {
    return m_Height;
  }

  // Special operations

  public void Clear( int a_Color ) {
    for (int x = 0; x < m_Width; ++x) {
      for (int y = 0; y < m_Height; ++y) {
        m_Buffer.setRGB(x, y, a_Color);
      }
    }
  }

  public void Print( String a_String, int x1, int y1, int color ) {

    for (int i = 0; i < a_String.length(); i++ ) {
      int pos = 0;
      if ((a_String.charAt(i) >= 'A') && (a_String.charAt(i) <= 'Z')) {
        pos = s_Transl[a_String.charAt(i) - ('A' - 'a')];
      } else {
        pos = s_Transl[a_String.charAt(i)];
      }
      String[] c = s_Font[pos];
      for (int v = 0; v < 5; v++ ) {
        for (int h = 0; h < 5; h++ ) {
          if (c[v].charAt(h) == 'o') {
            m_Buffer.setRGB(x1+h, y1+v, color);
          }
        }
      }
      x1 += 6;
    }
  }

  public void SetChar( int c, String[] c1 ) {
    s_Font[c] = new String[c1.length];
    for (int i = 0; i < c1.length; ++i) {
      s_Font[c][i] = c1[i];
    }
  }

  public void InitCharset() {
    s_Font = new String[51][];
    SetChar( 0, new String[] {":ooo:", "o:::o", "ooooo", "o:::o", "o:::o"} );
    SetChar( 1, new String[] {"oooo:", "o:::o", "oooo:", "o:::o", "oooo:"} );
    SetChar( 2, new String[] {":oooo", "o::::", "o::::", "o::::", ":oooo"} );
    SetChar( 3, new String[] {"oooo:", "o:::o", "o:::o", "o:::o", "oooo:"} );
    SetChar( 4, new String[] {"ooooo", "o::::", "oooo:", "o::::", "ooooo"} );
    SetChar( 5, new String[] {"ooooo", "o::::", "ooo::", "o::::", "o::::"} );
    SetChar( 6, new String[] {":oooo", "o::::", "o:ooo", "o:::o", ":ooo:"} );
    SetChar( 7, new String[] {"o:::o", "o:::o", "ooooo", "o:::o", "o:::o"} );
    SetChar( 8, new String[] {"::o::", "::o::", "::o::", "::o::", "::o::"} );
    SetChar( 9, new String[] {":::o:", ":::o:", ":::o:", ":::o:", "ooo::"} );
    SetChar(10, new String[] {"o::o:", "o:o::", "oo:::", "o:o::", "o::o:"} );
    SetChar(11, new String[] {"o::::", "o::::", "o::::", "o::::", "ooooo"} );
    SetChar(12, new String[] {"oo:o:", "o:o:o", "o:o:o", "o:::o", "o:::o"} );
    SetChar(13, new String[] {"o:::o", "oo::o", "o:o:o", "o::oo", "o:::o"} );
    SetChar(14, new String[] {":ooo:", "o:::o", "o:::o", "o:::o", ":ooo:"} );
    SetChar(15, new String[] {"oooo:", "o:::o", "oooo:", "o::::", "o::::"} );
    SetChar(16, new String[] {":ooo:", "o:::o", "o:::o", "o::oo", ":oooo"} );
    SetChar(17, new String[] {"oooo:", "o:::o", "oooo:", "o:o::", "o::o:"} );
    SetChar(18, new String[] {":oooo", "o::::", ":ooo:", "::::o", "oooo:"} );
    SetChar(19, new String[] {"ooooo", "::o::", "::o::", "::o::", "::o::"} );
    SetChar(20, new String[] {"o:::o", "o:::o", "o:::o", "o:::o", ":oooo"} );
    SetChar(21, new String[] {"o:::o", "o:::o", ":o:o:", ":o:o:", "::o::"} );
    SetChar(22, new String[] {"o:::o", "o:::o", "o:o:o", "o:o:o", ":o:o:"} );
    SetChar(23, new String[] {"o:::o", ":o:o:", "::o::", ":o:o:", "o:::o"} );
    SetChar(24, new String[] {"o:::o", "o:::o", ":oooo", "::::o", ":ooo:"} );
    SetChar(25, new String[] {"ooooo", ":::o:", "::o::", ":o:::", "ooooo"} );
    SetChar(26, new String[] {":ooo:", "o::oo", "o:o:o", "oo::o", ":ooo:"} );
    SetChar(27, new String[] {"::o::", ":oo::", "::o::", "::o::", ":ooo:"} );
    SetChar(28, new String[] {":ooo:", "o:::o", "::oo:", ":o:::", "ooooo"} );
    SetChar(29, new String[] {"oooo:", "::::o", "::oo:", "::::o", "oooo:"} );
    SetChar(30, new String[] {"o::::", "o::o:", "ooooo", ":::o:", ":::o:"} );
    SetChar(31, new String[] {"ooooo", "o::::", "oooo:", "::::o", "oooo:"} );
    SetChar(32, new String[] {":oooo", "o::::", "oooo:", "o:::o", ":ooo:"} );
    SetChar(33, new String[] {"ooooo", "::::o", ":::o:", "::o::", "::o::"} );
    SetChar(34, new String[] {":ooo:", "o:::o", ":ooo:", "o:::o", ":ooo:"} );
    SetChar(35, new String[] {":ooo:", "o:::o", ":oooo", "::::o", ":ooo:"} );
    SetChar(36, new String[] {"::o::", "::o::", "::o::", ":::::", "::o::"} );
    SetChar(37, new String[] {":ooo:", "::::o", ":::o:", ":::::", "::o::"} );
    SetChar(38, new String[] {":::::", ":::::", "::o::", ":::::", "::o::"} );
    SetChar(39, new String[] {":::::", ":::::", ":ooo:", ":::::", ":ooo:"} );
    SetChar(40, new String[] {":::::", ":::::", ":::::", ":::o:", "::o::"} );
    SetChar(41, new String[] {":::::", ":::::", ":::::", ":::::", "::o::"} );
    SetChar(42, new String[] {":::::", ":::::", ":ooo:", ":::::", ":::::"} );
    SetChar(43, new String[] {":::o:", "::o::", "::o::", "::o::", ":::o:"} );
    SetChar(44, new String[] {"::o::", ":::o:", ":::o:", ":::o:", "::o::"} );
    SetChar(45, new String[] {":::::", ":::::", ":::::", ":::::", ":::::"} );
    SetChar(46, new String[] {"ooooo", "ooooo", "ooooo", "ooooo", "ooooo"} );
    SetChar(47, new String[] {"::o::", "::o::", ":::::", ":::::", ":::::"} ); // Tnx Ferry
    SetChar(48, new String[] {"o:o:o", ":ooo:", "ooooo", ":ooo:", "o:o:o"} );
    SetChar(49, new String[] {"::::o", ":::o:", "::o::", ":o:::", "o::::"} );
    String c = "abcdefghijklmnopqrstuvwxyz0123456789!?:=,.-() #'*/";
    s_Transl = new int[256];
    for (int i = 0; i < 256; i++ ) {
      s_Transl[i] = 45;
    }
    for (int i = 0; i < 50; i++ ) {
      s_Transl[c.charAt(i)] = i;
    }
  }

  // Attributes
  private BufferedImage m_Buffer;
  private int m_Width, m_Height;

  // Static attributes for the buildin font
  private String[][] s_Font;
  private int[] s_Transl;
}


